package genius.core.uncertainty;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import genius.core.Bid;
import genius.core.bidding.BidDetails;
import genius.core.bidding.BidDetailsSorterUtility;
import genius.core.boaframework.SortedOutcomeSpace;

public class ComparisonGenerator {
	private SortedOutcomeSpace outcomeSpace;

	public ComparisonGenerator(SortedOutcomeSpace outcomeSpace) {
		this.outcomeSpace = outcomeSpace;
	}

	public BidRanking generateComparisonsByUncertaintyPercentage(
			double uncertaintyPercentage) {
		int amountOfOutcomes = (int) (uncertaintyPercentage
				* (outcomeSpace.getAllOutcomes().size()));
		return generateRankingByAmount(amountOfOutcomes);
	}

	/**
	 * 
	 * @param amountOfOutcomes
	 *            number of outcomes needed.
	 * @return amountOfOutcomes randomly picked bids, sorted from low to high
	 *         utility,
	 */
	public BidRanking generateRankingByAmount(int amountOfOutcomes) {
		List<BidDetails> selectedBidsWithUtilities = new ArrayList<BidDetails>();
		List<BidDetails> allBids = outcomeSpace.getAllOutcomes();
		Collections.shuffle(allBids);
		selectedBidsWithUtilities = allBids.stream()
				.filter(bid -> allBids.indexOf(bid) < amountOfOutcomes)
				.collect(Collectors.toList());
		// from low to high
		Collections.sort(selectedBidsWithUtilities,
				new BidDetailsSorterUtility().reversed());
		List<Bid> bids = new ArrayList<Bid>();
		selectedBidsWithUtilities.stream()
				.forEach(bid -> bids.add(bid.getBid()));
		return new BidRanking(bids);
	}

	public List<OutcomeComparison> generateComparisonsFromRankedOutcomeSet(
			List<BidDetails> selectedBids) {
		List<OutcomeComparison> comparisons = new ArrayList<OutcomeComparison>();
		for (int i = 0; i < selectedBids.size() - 1; i++) {
			comparisons.add(new OutcomeComparison(selectedBids.get(i),
					selectedBids.get(i + 1)));
		}
		return comparisons;
	}

	public List<OutcomeComparison> generateComparisonsWithError(int amount,
			double error) {
		List<OutcomeComparison> comparisons = generateRankingByAmount(amount)
				.getPairwiseComparisons();
		int wrongComparisonsAmount = (int) (error * comparisons.size());
		for (int i = 0; i < wrongComparisonsAmount; i++) {
			if (comparisons.get(i).getComparisonResult() == -1)
				comparisons.get(i).setComparisonResult(+1);
			else
				comparisons.get(i).setComparisonResult(-1);
		}
		return comparisons;
	}

	public SortedOutcomeSpace getOutcomeSpace() {
		return outcomeSpace;
	}

	public List<BidDetails> getAllBids() {
		return outcomeSpace.getAllOutcomes();
	}
}
